/**
 * Created by tonyjiang on 16/3/15.
 */
import env from 'chanjet-env-check';
import PluginManager from './PluginManager'
import NativeConfig from './native-config';
import EventEmitter2 from 'eventemitter2';

/**
 * 应用预处理类 , 在应用加载前 , 解决各种环境检测 , nativeAPI状态.
 * @author TonyJiang
 */
class Mutants{

    static EVENT_NAME = {
        READY: 'ready',
        RESUME: 'resume',
        SUSPEND: 'suspend'
    }

    emitter = new EventEmitter2();

    constructor(options = {}){

        /**
         * 环境对象
         * @type {EnvClass}
         * @typeof {}
         *
         * @property browser {String} 当前浏览器 , 取值范围请参考constant.browser
         * @property os {String}  当前操作系统 , 取值范围请参考constant.os
         * @property platform {String}  当前平台 , 取值范围请参考constant.platform
         *
         * @property constant {JSONObject} 环境常量 , 用来当作环境判断的对比值
         *
         *      @property constant.browser {JSONObject} 浏览器常量
         *          @property constant.browser.chrome {String} chrome浏览器
         *          @property constant.browser.firefox {String} firefox浏览器
         *          @property constant.browser.ie {String} ie浏览器
         *          @property constant.browser.opera {String} opera浏览器
         *          @property constant.browser.safari {String} safari浏览器
         *          @property constant.browser.unknown {String} unknown浏览器
         *
         *      @property constant.os {JSONObject} 操作系统常量
         *          @property constant.os.android {String}  安卓操作系统
         *          @property constant.os.ios {String}  ios操作系统
         *          @property constant.os.computer {String}  PC机
         *          @property constant.os.mobile {String}  手机
         *
         *      @property constant.platform {JSONObject} 平台常量
         *          @property constant.platform.chanjet {String} 畅捷通平台
         *          @property constant.platform.weixin {String} 微信平台
         *          @property constant.platform.browser {String} 浏览器 , 一般是开发环境
         *
         *
         */
        this._env = env;

        /**
         * 准备工作是否ready
         * @type {boolean}
         * @private
         */
        this._isReady = false;


        this.config(options);


    }

    init(){
        this._ready();
    }

    config(options){
        /**
         * 设置参数
         * @type {JSONObject}
         *  @property plugins {Array}   所需要插件, 需要使用require返回插件class
         *  @property auth  认证信息
         *      @property auth.chanjet {JSONObject}  chanjet平台认证信息
         *      @property auth.weixin {JSONObject}   微信平台认证信息 , 详见微信开发文档
         *          @property auth.weixin.appId {String}
         *          @property auth.weixin.timestamp {String}
         *          @property auth.weixin.nonceStr {String}
         *          @property auth.weixin.signature {String}
         *
         * @example
         * options = {
         *      plugins : [
         *          require('chanjet-plugin-base'),
         *          require('chanjet-plugin-sms'),
         *          require('chanjet-plugin-share')
         *      ],
         *
         *      auth : {
         *          chanjet : {},
         *          weixin : {
         *              appId : 'insert appId here',
         *              timestamp : '1458198827123',
         *              nonceStr : 'dwqq2321321',
         *              signature : 'qq2233'
         *          }
         *      }
         * }
         */
        this._options = Object.assign({} , options);



    }

    /**
     * 等待环境ready , 成功后回调callback
     * @param {Function} callback 成功后执行的回调
     */
    ready(callback){
        if(this._isReady){
            callback();
        }else{
            this.emitter.on(Mutants.EVENT_NAME.READY, callback);
        }
    }

    /**
     * 从后台切回来时触发
     * @param {Function} callback
     */
    resume(callback){
        this.emitter.on(Mutants.EVENT_NAME.RESUME, callback);
    }

    suspend(callback){
        this.emitter.on(Mutants.EVENT_NAME.SUSPEND, callback);
    }

    /**
     * 退出当前APP ,暂时不做回调处理
     */
    exit(){
        cordova.exec(
            ()=>{} , 
            ()=>{} , 
            'Destroy' , 'destroy' , []
        );
    }

    /**
     * 具体处理ready方法, 由实现类重写
     * @private
     */
    _ready(){}

    get env(){
        return this._env;
    }

    get constant(){
        return this._env.constant;
    }

    get plugin(){
        return PluginManager;
    }

    get nativeConfig() {
        return NativeConfig.instance;
    }

    //为了android提供, 检测当前是否为restore状态
    get isRestore(){
        return false;
    }

    //为了android提供, 获取restore状态下上次plugin的数据
    get restorePluginData(){
        return {};
    }

}




export default Mutants

